@@ -10,6 +10,7 @@
 
     Sao.Session = Sao.class_(Object, {
         init: function(database, login) {
+            this.login_service = null;
             this.user_id = null;
             this.session = null;
             this.cache = new Cache();
@@ -42,19 +43,19 @@
                     'params': [login, parameters, Sao.i18n.getlang()]
                 };
             };
-            new Sao.Login(func, this).run().then(function(result) {
+            new Sao.Login(func, this).run().then(result => {
                 this.login = login;
                 this.user_id = result[0];
                 this.session = result[1];
                 this.store();
                 this.renew_device_cookie();
                 dfd.resolve();
-            }.bind(this), function() {
+            }, () => {
                 this.user_id = null;
                 this.session = null;
                 this.store();
                 dfd.reject();
-            }.bind(this));
+            });
             return dfd.promise();
         },
         do_logout: function() {
@@ -93,10 +94,10 @@
             };
             this.reset_context();
             var prm = Sao.rpc(args, this);
-            return prm.then(function(context) {
+            return prm.then(context => {
                 jQuery.extend(this.context, context);
                 this.store_context();
-            }.bind(this));
+            });
         },
         reset_context: function() {
             this.context = {
@@ -124,6 +125,7 @@
                 if (session_data !== null) {
                     session_data = JSON.parse(session_data);
                     if (!this.login || this.login == session_data.login) {
+                        this.login_service = session_data.login_service;
                         this.login = session_data.login;
                         this.user_id = session_data.user_id;
                         this.session = session_data.session;
@@ -156,7 +158,7 @@
                 method: 'model.res.user.device.renew',
                 params: [device_cookie, {}],
             }, this);
-            renew_prm.done(function(result) {
+            renew_prm.done(result => {
                 device_cookies = JSON.parse(
                     localStorage.getItem('sao_device_cookies'));
                 if (!device_cookies) {
@@ -168,25 +170,57 @@
                 device_cookies[this.database][this.login] = result;
                 localStorage.setItem(
                     'sao_device_cookies', JSON.stringify(device_cookies));
-            }.bind(this));
+            });
+            renew_prm.fail(() => {
+                Sao.Logger.error("Cannot renew device cookie");
+            });
         }
     });
 
+    Sao.Session.server_version = function() {
+        var timeoutID = Sao.common.processing.show();
+        return jQuery.ajax({
+            'contentType': 'application/json',
+            'data': JSON.stringify({
+                'id': 0,
+                'method': 'common.server.version',
+                'params': []
+            }),
+            'dataType': 'json',
+            'url': '/',
+            'type': 'post',
+            'complete': [function() {
+                Sao.common.processing.hide(timeoutID);
+            }]
+        }).then(function(data) {
+            return data.result;
+        });
+    };
+
     Sao.Session.login_dialog = function() {
-        var dialog = new Sao.Dialog(Sao.i18n.gettext('Login'), 'lg');
+        var dialog = new Sao.Dialog(
+            Sao.i18n.gettext("Login"), 'login-dialog', 'md', true,
+            Sao.__version__);
         dialog.database_select = jQuery('<select/>', {
             'class': 'form-control',
-            'id': 'database'
+            'id': 'database',
+            'name': 'database',
         }).hide();
         dialog.database_input = jQuery('<input/>', {
             'class': 'form-control',
-            'id': 'database'
+            'id': 'database',
+            'name': 'database',
         }).hide();
         dialog.login_input = jQuery('<input/>', {
             'class': 'form-control',
             'id': 'login',
-            'placeholder': Sao.i18n.gettext('User name')
+            'name': 'login',
         });
+        dialog.button = jQuery('<button/>', {
+            'class': 'btn btn-primary btn-block',
+            'type': 'submit',
+            'title': Sao.i18n.gettext("Login"),
+        }).text(' ' + Sao.i18n.gettext("Login"));
         dialog.body.append(jQuery('<div/>', {
             'class': 'form-group'
         }).append(jQuery('<label/>', {
@@ -194,19 +228,22 @@
             'for': 'database'
         }).text(Sao.i18n.gettext('Database')))
         .append(dialog.database_select)
-        .append(dialog.database_input)
-        ).append(jQuery('<div/>', {
+        .append(dialog.database_input))
+        .append(jQuery('<div/>', {
+            'class': 'panel panel-default',
+        })
+        .append(jQuery('<div/>', {
+            'class': 'panel-body',
+        })
+        .append(jQuery('<div/>', {
             'class': 'form-group'
         }).append(jQuery('<label/>', {
             'class': 'control-label',
             'for': 'login'
         }).text(Sao.i18n.gettext('User name')))
         .append(dialog.login_input)
-        );
-        dialog.button = jQuery('<button/>', {
-            'class': 'btn btn-primary',
-            'type': 'submit'
-        }).text(' ' + Sao.i18n.gettext("Login")).appendTo(dialog.footer);
+        )
+        .append(dialog.button)));
         return dialog;
     };
 
@@ -232,7 +269,11 @@
                 });
         };
 
-        var ok_func = function() {
+        var disable_form = function(disabled=true) {
+            dialog.body.find('input,select,button').prop('disabled', disabled);
+        };
+
+        var login = function() {
             var login = dialog.login_input.val();
             var database = database || dialog.database_select.val() ||
                 dialog.database_input.val();
@@ -242,33 +283,67 @@
                 return;
             }
             dialog.button.focus();
-            dialog.button.prop('disabled', true);
-            dialog.modal.modal('hide');
+            disable_form();
             session.database = database;
             session.login = login;
             session.restore();
             (session.session ? jQuery.when() : session.do_login())
                 .then(function() {
+                    dialog.modal.modal('hide');
                     dfd.resolve(session);
                     dialog.modal.remove();
                     if (database_url() != database) {
                         window.location = '#' + database;
                     }
                 }, function() {
-                    dialog.button.prop('disabled', false);
-                    dialog.modal.modal('show');
+                    disable_form(false);
                     empty_field().closest('.form-group').addClass('has-error');
                     empty_field().first().focus();
                 });
         };
 
+        var login_service = function(evt) {
+            var database = database || dialog.database_select.val() ||
+                dialog.database_input.val();
+            dialog.modal.find('.has-error').removeClass('has-error');
+            if (!database) {
+                dialog.database_select
+                    .closest('.form-group').addClass('has-error');
+                return;
+            }
+            disable_form();
+            var host = window.location.protocol + '//' + window.location.host;
+            var next = new URL(host + '/');
+            next.searchParams.append('login_service', evt.data);
+            var url = new URL(host + '/' + database + evt.data);
+            url.searchParams.append('next', next.href);
+            var service_window = window.open(url.href, '_blank', 'popup=1');
+            const timer = window.setInterval(() => {
+                if (service_window.closed) {
+                    window.clearInterval(timer);
+                    session.database = database;
+                    session.restore();
+                    if (session.session) {
+                        dfd.resolve(session);
+                        dialog.modal.remove();
+                        if (database_url() != database) {
+                            window.location = '#' + database;
+                        }
+                    } else {
+                        disable_form(false);
+                        empty_field().first().focus();
+                    }
+                }
+            }, 500);
+        };
+
         dialog.modal.modal({
             backdrop: false,
             keyboard: false
         });
         dialog.modal.find('form').unbind().submit(function(e) {
-            ok_func();
             e.preventDefault();
+            login();
         });
         dialog.modal.on('shown.bs.modal', function() {
             empty_field().first().focus();
@@ -282,12 +357,12 @@
                 el = dialog.database_input;
             } else {
                 el = dialog.database_select;
-                databases.forEach(function(database) {
+                for (const database of databases) {
                     el.append(jQuery('<option/>', {
                         'value': database,
                         'text': database
                     }));
-                });
+                }
             }
             el.prop('readonly', databases.length == 1);
             el.show();
@@ -295,6 +370,24 @@
         }, function() {
             dialog.database_input.show();
         });
+
+        jQuery.when(Sao.Authentication.services()).then(function(services) {
+            if (services.length) {
+                var panel_body = jQuery('<div/>', {
+                    'class': 'panel-body',
+                }).append(jQuery('<p/>')
+                    .text(Sao.i18n.gettext("Login with")));
+                dialog.body.append(jQuery('<div/>', {
+                    'class': 'panel panel-default',
+                }).append(panel_body));
+                for (const [name, url] of services) {
+                    panel_body.append(jQuery('<button/>', {
+                        'class': 'btn btn-block btn-default',
+                        'type': 'button',
+                    }).text(name).click(url, login_service));
+                }
+            }
+        });
         return dfd.promise();
     };
 
@@ -303,11 +396,36 @@
             return session.prm;
         }
         var dfd = jQuery.Deferred();
+        session.session = null;
         session.prm = dfd.promise();
-        session.do_login().then(dfd.resolve, function() {
-            Sao.logout();
-            dfd.reject();
-        }).done(function () {
+        if (!session.login_service) {
+            session.do_login().then(dfd.resolve, function() {
+                Sao.logout();
+                dfd.reject();
+            });
+        } else {
+            session.unstore();
+            var host = window.location.protocol + '//' + window.location.host;
+            var next = new URL(host + '/');
+            next.searchParams.append('login_service', session.login_service);
+            next.searchParams.append('renew', session.user_id);
+            var url = new URL(host + '/' + session.database + session.login_service);
+            url.searchParams.append('next', next.href);
+            var service_window = window.open(url.href, '_blank', 'popup=1');
+            const timer = window.setInterval(() => {
+                if (service_window.closed) {
+                    window.clearInterval(timer);
+                    session.restore();
+                    if (session.session) {
+                        dfd.resolve();
+                    } else {
+                        Sao.logout();
+                        dfd.reject();
+                    }
+                }
+            }, 500);
+        }
+        dfd.done(function() {
             Sao.Bus.listen();
         });
         return session.prm;
@@ -320,10 +438,7 @@
             this.func = func;
             this.session = session || Sao.Session.current_session;
         },
-        run: function(parameters) {
-            if (parameters === undefined) {
-                parameters = {};
-            }
+        run: function(parameters={}) {
             var dfd = jQuery.Deferred();
             var timeoutID = Sao.common.processing.show();
             var data = this.func(parameters);
@@ -358,20 +473,22 @@
                             Sao.i18n.gettext('Too many requests. Try again later.'),
                             'tryton-error').always(dfd.resolve);
                     } else if (data.error[0].startsWith('404')) {
-                        dfd.reject();
+                        Sao.common.message.run(
+                            Sao.i18n.gettext("Not found."),
+                            'tryton-error').always(dfd.reject);
                     } else if (data.error[0] != 'LoginException') {
-                        Sao.common.error.run(data.error[0], data.error[1]);
-                        dfd.reject();
+                        Sao.common.error.run(data.error[0], data.error[1])
+                            .always(dfd.reject);
                     } else {
                         var args = data.error[1];
                         var name = args[0];
                         var message = args[1];
                         var type = args[2];
-                        this['get_' + type](message).then(function(value) {
+                        this['get_' + type](message, name).then(value => {
                             parameters[name] = value;
                             return this.run(parameters).then(
                                     dfd.resolve, dfd.reject);
-                        }.bind(this), dfd.reject);
+                        }, dfd.reject);
                     }
                 } else {
                     dfd.resolve(data.result);
@@ -385,20 +502,23 @@
                     Sao.common.message.run(
                         Sao.i18n.gettext('Too many requests. Try again later.'),
                         'tryton-error').always(dfd.resolve);
+                } else if (query.status == 404) {
+                    Sao.common.message.run(
+                        Sao.i18n.gettext("Not found."),
+                        'tryton-error').always(dfd.reject);
                 } else {
-                    Sao.common.error.run(status_, error);
-                    dfd.reject();
+                    Sao.common.error.run(status_, error).always(dfd.reject);
                 }
             };
             ajax_prm.done(ajax_success.bind(this));
             ajax_prm.fail(ajax_error.bind(this));
             return dfd.promise();
         },
-        get_char: function(message) {
-            return Sao.common.ask.run(message);
+        get_char: function(message, name) {
+            return Sao.common.ask.run(message, name);
         },
-        get_password: function(message) {
-            return Sao.common.ask.run(message, false);
+        get_password: function(message, name) {
+            return Sao.common.ask.run(message, name, false);
         },
     });
 
@@ -426,6 +546,7 @@
                 delete this.store[prefix][key];
                 return undefined;
             }
+            Sao.Logger.info("(cached)", prefix, key);
             return Sao.rpc.convertJSONObject(jQuery.parseJSON(data.value));
         },
         clear: function(prefix) {
@@ -458,4 +579,29 @@
             return data.result;
         });
     };
+
+    Sao.Authentication = {};
+
+    Sao.Authentication.services = function() {
+        var timeoutID = Sao.common.processing.show();
+        return jQuery.ajax({
+            'contentType': 'application/json',
+            'data': JSON.stringify({
+                'id': 0,
+                'method': 'common.authentication.services',
+                'params': []
+            }),
+            'dataType': 'json',
+            'url': '/',
+            'type': 'post',
+            'complete': [function() {
+                Sao.common.processing.hide(timeoutID);
+            }]
+        }).then(function(data) {
+            Sao.Authentication.services = function() {
+                return data.result;
+            };
+            return data.result;
+        });
+    };
 }());
